<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
$plugin = EOS_DP_PLUGIN_BASE_NAME;
define( 'EOS_DP_DOCUMENTATION_URL','https://freesoul-deactivate-plugins.com/documentation/' );
if( eos_dp_is_fdp_page() ){
	$dir = EOS_DP_PLUGIN_DIR.'/admin/templates/';
	require_once $dir.'partials/eos-dp-navigation.php';
	require_once $dir.'partials/eos-dp-table-head.php';
	require_once $dir.'partials/eos-dp-footer.php';	
	foreach( array(
		array( array( 'eos_dp_home','eos_dp_menu' ),'pages/eos-dp-singles.php' ),
		array( array( 'eos_dp_by_post_type' ),'pages/eos-dp-post-type.php' ),
		array( array( 'eos_dp_by_archive' ),'pages/eos-dp-archive.php' ),
		array( array( 'eos_dp_by_term_archive' ),'pages/eos-dp-terms-archive.php' ),
		array( array( 'eos_dp_mobile' ),'pages/eos-dp-mobile.php' ),
		array( array( 'eos_dp_search' ),'pages/eos-dp-search.php' ),
		array( array( 'eos_dp_url' ),'pages/eos-dp-url.php' ),
		array( array( 'eos_dp_admin_url' ),'pages/eos-dp-backend-url.php' ),
		array( array( 'eos_dp_admin' ),'pages/eos-dp-backend.php' ),
		array( array( 'eos_dp_code_risk' ),'pages/eos-dp-code-risk.php' ),
		array( array( 'eos_dp_firing_order' ),'pages/eos-dp-firing-order.php' ),
		array( array( 'eos_dp_help' ),'pages/eos-dp-help.php' )
	) as $arr ){
		if( isset( $_GET['page'] ) && in_array( $_GET['page'],$arr[0] ) ){
			require_once $dir.$arr[1];
		}
	}
	if( defined( 'FDP_BETA_VERSION' ) && FDP_BETA_VERSION && defined( 'FDP_REMOTE_HELP' ) && FDP_REMOTE_HELP ){
		require_once $dir.'pages/eos-dp-advanced-support.php';
	}
}
else{
	add_action( 'admin_enqueue_scripts','eos_dp_optimization_plugins_js' );
}	

//It enqueues JS integration for popular cache plugins	
function eos_dp_optimization_plugins_js(){
	if( !current_user_can( 'activate_plugins' ) ) return;
	$user_opts = get_site_option( 'eos_dp_user_options' );
	$user = wp_get_current_user();
	if( !is_object( $user ) ) return;
	if( isset( $user_opts[sanitize_key( $user->user_login )] ) ){
		$user_opts = $user_opts[sanitize_key( $user->user_login )];
		if( isset( $user_opts['concatenation'] ) && 'never_again' === $user_opts['concatenation'] ) return;
	}
	$pages = array(
		'WPFC_MAIN_PATH' => array( 'page','wpfastestcacheoptions' ),
		'W3TC' =>  array( 'page','w3tc_minify' ),
		'WP_ROCKET_VERSION' =>  array( 'page','wprocket' ),
		'AUTOPTIMIZE_PLUGIN_VERSION' =>  array( 'page','autoptimize' ),
		'LSCWP_DIR' => array( 'page','lscache-settings' ),
		'LSCWP_DIR' => array( 'page','lscache-settings' ),
	);
	$require_ajax = false;
	$plugins_constants = eos_dp_optimization_plugins_info();
	foreach( $plugins_constants as $constant => $arr ){
		if( defined( $constant ) ){
			$getArg = $pages[$constant];
			if( isset( $_GET[$getArg[0]] ) && $getArg[1] === $_GET[$getArg[0]] ){
				$active_cache_plugins = $plugins_constants[$constant];
				wp_enqueue_script( 'eos-dp-optimization-integration',EOS_DP_PLUGIN_URL.'/admin/js/optimization-integration.js',array( 'jquery' ),EOS_DP_VERSION,true );
				wp_localize_script( 
					'eos-dp-optimization-integration',
					'eos_dp_optimization_integration',
					array( 
						'active_plugins' => json_encode( $active_cache_plugins ),
						'msg_http1' => __( "FREESOUL DEACTIVATE PLUGINS. Concatenating resources may reduce the benefit of disabling plugins.","eos-dp" ),
						'msg_http2' => __( "FREESOUL DEACTIVATE PLUGINS. Concatenating resources will probably reduce the benefit of disabling plugins.","eos-dp" ),
						'msg_unknown' => __( "FREESOUL DEACTIVATE PLUGINS. Concatenating resources may reduce the benefit of disabling plugins.","eos-dp" ),
						'never_again' => __( "Close and don't show again","eos-dp" ),
						'close' => __( "Close","eos-dp" ),
						'ajax_url' => admin_url( 'admin-ajax.php' ),
						'nonce' => wp_create_nonce( 'eos_dp_never_again_msg_user' ),
						'learn_more' => __( "Learn more","eos-dp" ),
						'learn_more_url' => 'https://freesoul-deactivate-plugins.com/should-i-still-combine-static-resources-with-http-2/',
					)
				);
				$require_ajax = true;
			}
		}
	}
}	

//It adds a settings link to the action links in the plugins page
add_filter( "plugin_action_links_$plugin", 'eos_dp_plugin_add_settings_link' );

//It redirects to the plugin settings page on successfully plugin activation
add_action( 'admin_init', 'eos_dp_redirect_to_settings' );

//It displays the admin notices
add_action( 'admin_notices','eos_dp_admin_notices',100 );

//It adds the plugin setting page under plugins menu
add_action( 'admin_menu','eos_dp_options_page',999 );

add_filter( 'admin_title', 'eos_dp_admin_page_title',99, 2 );
//It set the browser tab title depending the options page
function eos_dp_admin_page_title( $title,$sep ){
	$titles = array(
		'eos_dp_by_post_type' => __( 'Disable plugins | Post Types','eos-dp' ),
		'eos_dp_menu' => __( 'Disable plugins | Singles','eos-dp' ),
		'eos_dp_by_archive' => __( 'Disable plugins | Archives','eos-dp' ),
		'eos_dp_by_term_archive' => __( 'Disable plugins | Terms Archives','eos-dp' ),
		'eos_dp_mobile' => __( 'Disable plugins | Mobile','eos-dp' ),
		'eos_dp_search' => __( 'Disable plugins | Search','eos-dp' ),
		'eos_dp_url' => __( 'Disable plugins | URL','eos-dp' ),
		'eos_dp_admin_url' => __( 'Disable plugins | Admin URL','eos-dp' ),
		'eos_dp_admin' => __( 'Disable plugins | Back-end','eos-dp' ),
		'eos_dp_documentation' => __( 'Freesoul Deactivate Plugins Documentation','eos-dp' ),
	);
	if( isset( $_GET['page'] ) && in_array( $_GET['page'],array_keys( $titles ) ) ){	
		return esc_html( $titles[$_GET['page']] );
	}
	if( isset( $_GET['eos_dp_info'] ) && 'true' == $_GET['eos_dp_info'] ){
		if( isset( $_GET['plugin'] ) ){
			return __( 'Plugin Details','eos-dp' );
		}
	}
	return $title;
}

//Remove other admin notices on the settings pages
function eos_dp_remove_other_admin_notices(){
	remove_all_actions( 'admin_notices' );
	remove_all_actions( 'all_admin_notices' );
	add_action( 'admin_notices','eos_dp_admin_notices' );
}

if( isset( $_GET['page'] ) && in_array( $_GET['page'],array( 'eos_dp_admin','eos_dp_ajax' ) ) ){
	add_action( 'eos_dp_after_table_head_columns','eos_dp_add_theme_to_table_head' );
}
//It adds the theme column in the table header
function eos_dp_add_theme_to_table_head(){
	$theme = wp_get_theme();
	if( !is_object( $theme ) ) return;
	$theme_name = strtoupper( $theme->get( 'TextDomain' ) );
	$theme_name_short = substr( $theme_name,0,28 );
	$theme_name_short = $theme_name === $theme_name_short ? $theme_name : strtoupper( $theme_name_short ).' ...';
	?>
	<th class="eos-dp-name-th eos-dp-name-th-theme">
		<div>
			<div id="eos-dp-theme-name" class="eos-dp-theme-name" data-theme="<?php echo esc_attr( $theme->get( 'TextDomain' ) ); ?>" title="<?php echo esc_attr( $theme_name ); ?>" data-path="<?php echo get_stylesheet_directory_uri(); ?>">
				<span><?php echo esc_html( $theme_name_short ); ?></span>
			</div>
			<div id="eos-dp-global-chk-col-wrp" class="eos-dp-global-chk-col-wrp">
				<div class="eos-dp-not-active-wrp"><input title="<?php printf( __( 'Activate/deactivate %s everywhere','eos-dp' ),esc_attr( $theme_name ) ); ?>" data-col="theme" class="eos-dp-global-chk-col" type="checkbox" /></div>
			</div>
		</div>
	</th>	
	<?php
}

add_action( 'eos_dp_pre_table_head','eos_dp_pro_nonces' );
//It displays the auto settings button and related messages
function eos_dp_pro_nonces(){
	wp_nonce_field( 'eos_dp_pro_auto_settings', 'eos_dp_pro_auto_settings' );
	wp_nonce_field( 'eos_dp_plugins_contributions', 'eos_dp_plugins_contributions' );
	wp_nonce_field( 'eos_dp_pro_errors_check', 'eos_dp_pro_errors_check' );
	wp_nonce_field( 'eos_dp_pro_gt_metrix_test', 'eos_dp_pro_gt_metrix_test' );

}

add_action( 'eos_dp_action_buttons','eos_dp_home_autosuggest_action_buttons',10 );

//It adds premium action buttons
function eos_dp_home_autosuggest_action_buttons(){
	if( isset( $_GET['eos_dp_home'] ) ) : ?>
	<a href="#" class="eos-dp-pro-autosettings" title="<?php _e( 'Suggest plugins','eos-dp' ); ?>"><span class="dashicons dashicons-plugins-checked"></span></a>
	<?php endif;
}

add_filter( 'admin_body_class', 'eos_dp_admin_body_class' );
// Adds the class to the body tag in the dashboard according the options page.
function eos_dp_admin_body_class( $classes ) {
	if( isset( $_GET['page'] ) && eos_dp_is_fdp_page() ){
		$classes .= ' eos-dp-'.esc_attr( $_GET['page'] );
		if( isset( $_GET['eos_dp_home'] ) && 'true' === $_GET['eos_dp_home'] ){
			$classes .= ' eos-dp-homepage';
		}
	}
	return $classes;
}

if( isset( $_GET['eos_dp_preview'] ) && isset( $_GET['js'] ) && 'off' === $_GET['js'] ){
	add_action( 'admin_head','eos_dp_disable_javascript',10 );		
}

add_action( 'admin_init','eos_dp_redirect_home_settings' );
function eos_dp_redirect_home_settings(){
	//Redirect to homepage settings
	if( isset( $_GET['eos_dp_home'] ) && 'true' === $_GET['eos_dp_home'] ){
		$show_on_front = eos_dp_get_option( 'show_on_front' );
		if( isset( $_GET['page'] ) && 'eos_dp_menu' === $_GET['page'] && 'posts' === $show_on_front ){
			wp_safe_redirect( admin_url( 'admin.php?page=eos_dp_by_archive&eos_dp_home=true' ) );
			exit;
		}
		if( isset( $_GET['page'] ) && 'eos_dp_by_archive' === $_GET['page'] && 'page' === $show_on_front ){
			if( absint( eos_dp_get_option( 'page_on_front' ) ) > 0 ){
				wp_safe_redirect( admin_url( 'admin.php?page=eos_dp_menu&eos_dp_home=true' ) );
				exit;
			}		
		}
	}
}