<?php
/**
 *  File required by options.php that includes all the functions needed for admin ajax requests
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}
add_action("wp_ajax_eos_dp_send_remote_help_request", "eos_dp_send_remote_help_request");
//Send remote help request
function eos_dp_send_remote_help_request(){
	eos_dp_check_intentions_and_rights( 'eos_dp_advanced_support' );
	include_once( ABSPATH . 'wp-admin/includes/theme-install.php' );
	$active_theme = wp_get_theme();
	$theme_version = $active_theme->Version;
	$theme_template = $active_theme->Template;
	$post_types = array_keys( array_merge( array( 'page' => 'page' ),get_post_types( array( 'publicly_queryable' => true ) ) ) );
	$args = array(
		'publicly_queryable' => true,
		'post_type' => $post_types,
		'posts_per_page' => -1,
		'post_status' => 'public'
	);
	$query = new WP_Query( $args );
	$posts = $query->posts;
	$pages = array();
	foreach( $posts as $post ){
		if( is_object( $post ) && 'attachment' !== $post->post_type ){
			$pages[] = array( 'id' => $post->ID,'url' => get_permalink( $post->ID ),'post_type' => $post->post_type );
		}
	}
	global $wpdb;
	$data = array(
		'email' => sanitize_email( $_POST['remote_help_email'] ),
		'remote_help_username' => esc_attr( $_POST['remote_help_username'] ),
		'remote_help_password' => esc_attr( $_POST['remote_help_password'] ),
		'site_url' => site_url(),
		'home_url' => home_url(),
		'php_version' => function_exists( 'phpversion' ) ? phpversion() : 'unknown',
		'mysql_version' => is_object( $wpdb ) ? $wpdb->db_version() : 'unknown',
		'permalink_structure' => get_option( 'permalink_structure' ),
		'pages' => $pages,
		'debug' => defined( 'WP_DEBUG' ) && WP_DEBUG,
		'debug_display' => defined( 'WP_DEBUG_DISPLAY' ) && WP_DEBUG_DISPLAY,
		'debug_log' => defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG,
		'wordpress_version' => get_bloginfo( 'version' ),
		'is_multisite' => is_multisite(),
		'memory_limit' => max( eos_let_to_num( WP_MEMORY_LIMIT ),eos_let_to_num( @ini_get( 'memory_limit' ) ) ),
		'hostname' => function_exists( 'gethostname' ) ? gethostname() : false,
		'active_plugins' => eos_dp_get_option( 'active_plugins' ),
		'active_theme' => $theme_template,
		'theme_version' => $theme_version,
		'has_child_theme' => is_child_theme(),
		'advanced_cache' => file_exists( WP_CONTENT_DIR.'/advanced-cache.php' ),
		'opts' => eos_dp_get_option( 'eos_dp_opts' ),
		'eos_dp_activation_info' => eos_dp_get_option( 'eos_dp_activation_info' ),
		'eos_dp_search' => eos_dp_get_option( 'eos_dp_search' ),
		'eos_dp_mobile' => eos_dp_get_option( 'eos_dp_mobile' ),
		'eos_dp_by_url' => eos_dp_get_option( 'eos_dp_by_url' ),
		'post_types' => $post_types
	);
	$url = 'https://freesoul-deactivate-plugins.com/remote-support/';
	$response = wp_remote_post( $url, array(
		'method'      => 'POST',
		'timeout'     => 45,
		'redirection' => 5,
		'httpversion' => '1.0',
		'blocking'    => true,
		'headers'     => array(),
		'body'        => $data,
		'cookies'     => array()
		)
	);
	 
	if ( is_wp_error( $response ) ) {
		$error_message = $response->get_error_message();
		echo 0;
		die();
	}
	echo 1;
	die();
	exit;
}
add_action("wp_ajax_eos_dp_save_advanced_help", "eos_dp_save_advanced_help");
//Saves general options
function eos_dp_save_advanced_help(){
	eos_dp_check_intentions_and_rights( 'eos_dp_advanced_support' );
	if( isset( $_POST['advanced_help_username'] ) && isset( $_POST['advanced_help_password'] ) && isset( $_POST['advanced_help_permission'] ) ){
		$opts = eos_dp_get_option( 'eos_dp_opts' );
		$opts['advanced_help_username'] = sanitize_text_field( $_POST['advanced_help_username'] );
		$opts['advanced_help_password'] = sanitize_text_field( $_POST['advanced_help_password'] );
		$opts['advanced_help_permission'] = sanitize_text_field( $_POST['advanced_help_permission'] );
		eos_dp_update_option( 'eos_dp_opts',$opts );
		echo 1;
		die();
	}
	echo 0;
	die();
}
add_action("wp_ajax_eos_dp_save_settings", "eos_dp_save_settings");
//Saves activation/deactivation settings for each post
function eos_dp_save_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_setts' );
	if( isset( $_POST['eos_dp_setts'] ) && !empty( $_POST['eos_dp_setts'] ) ){
		foreach( $_POST['eos_dp_setts'] as $post_id => $opts ){
			$post_id = absint( str_replace( 'post_id_','',$post_id ) );
			if( $post_id > 0 ){
				update_post_meta( $post_id,'_eos_deactive_plugins_key',sanitize_text_field( $opts ) );
			}
		}
		$opts = $_POST['eos_dp_setts'];
		if( isset( $opts['post_type'] ) ){
			$post_types_matrix = eos_dp_get_updated_plugins_table();
			$post_type = sanitize_key( $opts['post_type'] );
			$post_types_matrix_pt = $post_types_matrix[$post_type];
			if( isset( $post_types_matrix_pt[3] ) && is_array( $post_types_matrix_pt[3] ) && !empty( $post_types_matrix_pt[3] ) ){
				if( isset( $opts['ids_locked'] ) ){
					foreach( $opts['ids_locked'] as $id_locked ){
						if( !in_array( $id_locked,$post_types_matrix_pt[3] ) ){
							$post_types_matrix_pt[3] = array_merge( $post_types_matrix_pt[3],array( $id_locked ) );
						}
					}
				}
				if( isset( $opts['ids_unlocked'] ) ){
					foreach( $opts['ids_unlocked'] as $id_unlocked ){
						if( in_array( $id_unlocked,$post_types_matrix_pt[3] ) ){
							$post_types_matrix_pt[3] = array_diff( $post_types_matrix_pt[3],array( $id_unlocked ) );
						}
					}
				}
			}
			else{
				$post_types_matrix_pt[3] = $opts['ids_locked'];
				$post_types_matrix[$opts['post_type']] = $post_types_matrix_pt;
			}
			$post_types_matrix[$post_type] = $post_types_matrix_pt;
			eos_dp_update_option( 'eos_post_types_plugins',$post_types_matrix );
			
			
		}	
		eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
	}
	echo 1;
	die();
	exit;
}
add_action("wp_ajax_eos_dp_save_archives_settings", "eos_dp_save_archives_settings");
//Saves activation/deactivation settings for each archive
function eos_dp_save_archives_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_arch_setts' );
	if( isset( $_POST['eos_dp_setts_archives'] ) && !empty( $_POST['eos_dp_setts_archives'] ) ){
		$archiveSetts = $_POST['eos_dp_setts_archives'];
		foreach( $archiveSetts as $k => $v ){
			unset( $archiveSetts[$k] );
			$kArr = explode( '//',$k );
			if( isset( $kArr[1] ) ){
				$k = rtrim( $kArr[1],'/' );
			}
			$k = str_replace( '/','__',$k );
			$permalink_structure = eos_dp_get_option( 'permalink_structure' );
			if( false !== strpos( $permalink_structure,'%category%' ) && '%postname%' === basename( $permalink_structure )  && '.' === eos_dp_get_option( 'category_base' ) ){
				$k = str_replace( '__.__','__',$k );
			}
			$archiveSetts[sanitize_key( $k )] = sanitize_text_field( $v );
		}
		$currentOpts = eos_dp_get_option( 'eos_dp_archives' );
		if( null !== $currentOpts && !empty( $currentOpts ) && null !== $archiveSetts && !empty( $archiveSetts ) ){
			$archiveSetts = array_merge( $currentOpts,$archiveSetts );
		}
		eos_dp_update_option( 'eos_dp_archives',$archiveSetts );
		eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
	}
	echo 1;
	die();
	exit;
}
add_action("wp_ajax_eos_dp_save_post_type_settings", "eos_dp_save_post_type_settings");
//Saves activation/deactivation settings for each post type
function eos_dp_save_post_type_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_pt_setts' );
	if( isset( $_POST['eos_dp_pt_setts'] ) && !empty( $_POST['eos_dp_pt_setts'] ) ){
		$opts = eos_dp_get_updated_plugins_table();
		if( !is_array( $opts ) ){
			$opts = array();
		}		
		$eos_dp_pt_setts = json_decode( str_replace( '\\','',$_POST['eos_dp_pt_setts'] ),true );
		foreach( $eos_dp_pt_setts as $post_type => $data ){
			$opts_post_type = $opts[sanitize_key( $post_type )];
			$locked_ids = isset( $opts_post_type[3] ) ? $opts_post_type[3] : array();
			$opts[sanitize_key( $post_type )] = array( absint( $data[0] ),sanitize_text_field( $data[1] ),absint( $data[2] ),$locked_ids );
		}
		eos_dp_update_option( 'eos_post_types_plugins',$opts );
		eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
		echo 1;
		die();
	}
	echo 0;
	die();
}

add_action("wp_ajax_eos_dp_save_url_settings", "eos_dp_save_url_settings");
//Saves activation/deactivation settings by URL
function eos_dp_save_url_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_url_setts' );
	if( isset( $_POST['eos_dp_url_setts'] ) && !empty( $_POST['eos_dp_url_setts'] ) ){
		$opts = get_site_option( 'eos_dp_by_url' );
		$eos_url_setts = json_decode( str_replace( '\\','',$_POST['eos_dp_url_setts'] ),true );
		$n = 0;
		foreach( $eos_url_setts as $arr ){
			$eos_dp_url_setts[$n] = array(
					'url' => sanitize_text_field( $arr['url'] ),
					'plugins' => sanitize_text_field( $arr['plugins'] )
				);
			++$n;
		}
		eos_dp_update_option( 'eos_dp_by_url',$eos_dp_url_setts );
		eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
		echo 1;
		die();
	}
	echo 0;
	die();
}
add_action("wp_ajax_eos_dp_save_mobile_settings", "eos_dp_save_mobile_settings");
//Saves activation/deactivation settings for mobile
function eos_dp_save_mobile_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_mobile_setts' );
	if( isset( $_POST['eos_dp_mobile'] ) && !empty( $_POST['eos_dp_mobile'] ) ){
		$opts = array_filter( explode( ',',sanitize_text_field( $_POST['eos_dp_mobile'] ) ) );
		eos_dp_update_option( 'eos_dp_mobile',$opts );
		echo 1;
		die();
	}
	echo 0;
	die();
}
add_action("wp_ajax_eos_dp_save_search_settings", "eos_dp_save_search_settings");
//Saves activation/deactivation settings for search
function eos_dp_save_search_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_search_setts' );
	if( isset( $_POST['eos_dp_search'] ) && !empty( $_POST['eos_dp_search'] ) ){
		$opts = array_filter( explode( ',',sanitize_text_field( $_POST['eos_dp_search'] ) ) );
		eos_dp_update_option( 'eos_dp_search',$opts );
		echo 1;
		die();
	}
	echo 0;
	die();
}

add_action("wp_ajax_eos_dp_save_admin_url_settings", "eos_dp_save_admin_url_settings");
//Saves activation/deactivation settings by Admin URL
function eos_dp_save_admin_url_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_admin_url_setts' );
	if( isset( $_POST['eos_dp_admin_url_setts'] ) && !empty( $_POST['eos_dp_admin_url_setts'] ) ){
		$opts = get_site_option( 'eos_dp_by_admin_url' );
		$eos_url_setts = json_decode( str_replace( '\\','',$_POST['eos_dp_admin_url_setts'] ),true );
		$n = 0;
		foreach( $eos_url_setts as $arr ){
			$eos_dp_admin_url_setts[$n] = array(
					'url' => sanitize_text_field( $arr['url'] ),
					'plugins' => sanitize_text_field( $arr['plugins'] )
				);
			++$n;
		}
		eos_dp_update_option( 'eos_dp_by_admin_url',$eos_dp_admin_url_setts );
		if( isset( $_POST['theme_activation'] ) ){
			eos_dp_update_option( 'eos_dp_admin_url_theme',json_decode( str_replace( '\\','',sanitize_text_field( $_POST['theme_activation'] ) ),true ) );
		}		
		eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
		echo 1;
		die();
	}
	echo 0;
	die();
}

add_action( 'wp_ajax_eos_dp_save_admin_settings', 'eos_dp_save_admin_settings' );
//Saves admin options
function eos_dp_save_admin_settings(){
	eos_dp_check_intentions_and_rights( 'eos_dp_admin_setts' );
	$opts = json_decode( str_replace( '\\','',$_POST['eos_dp_admin_setts'] ),true );
	eos_dp_update_option( 'eos_dp_new_plugin_activated',false );
	eos_dp_update_option( 'eos_dp_admin_setts',array_map( 'sanitize_text_field',$opts ) );	
	if( isset( $_POST['admin_menus'] ) && isset( $_POST['icons'] ) && isset( $_POST['parent_titles'] ) ){
		$admin_menus = json_decode( stripslashes( $_POST['admin_menus'] ),true );
		$icons = $_POST['icons'];
		$parent_titles = $_POST['parent_titles'];
		$n = 0;
		foreach( $admin_menus as $key => &$admin_menu ){
			if( isset( $icons[$key] ) ){
				$admin_menu['icons'] = sanitize_text_field( $icons[$key] );
			}
			if( isset( $parent_titles[$key] ) ){
				$admin_menu['parent_title'] = sanitize_text_field( $parent_titles[$key] );
			}
			elseif( isset( $parent_titles['admin.php?page='.$key] ) ){
				$admin_menu['parent_title'] = sanitize_text_field( $parent_titles['admin.php?page='.$key] );
			}			
			++$n;
		}
		eos_dp_update_option( 'eos_dp_admin_menu',sanitize_text_field( json_encode( $admin_menus ) ) );
	}
	if( isset( $_POST['theme_activation'] ) ){
		eos_dp_update_option( 'eos_dp_admin_theme',json_decode( str_replace( '\\','',sanitize_text_field( $_POST['theme_activation'] ) ),true ) );
	}
	if( isset( $_POST['menu_in_topbar'] ) ){
		$opts = eos_dp_get_option_array( 'eos_dp_general_setts' );
		$opts['menu_in_topbar'] = sanitize_text_field( $_POST['menu_in_topbar'] );
		eos_dp_update_option( 'eos_dp_general_setts',$opts );
	}
	echo 1;
	die();
	exit;
}

add_action("wp_ajax_eos_dp_save_firing_order", "eos_dp_save_firing_order");
//Saves activation/deactivation settings for search
function eos_dp_save_firing_order(){
	eos_dp_check_intentions_and_rights( 'eos_dp_firing_order_setts' );
	if( isset( $_POST['eos_dp_plugins'] ) && !empty( $_POST['eos_dp_plugins'] ) ){
		$opts = array_map( 'sanitize_text_field',$_POST['eos_dp_plugins'] );	
		eos_dp_update_option( 'active_plugins',$opts );
		echo 1;
		die();
	}
	echo 0;
	die();
}
add_action("wp_ajax_eos_dp_preview", "eos_dp_preview");
//Prepare the transient for the preview
function eos_dp_preview(){
	$nonceStr = isset( $_POST['post_id'] ) ? 'eos_dp_setts' : 'eos_dp_arch_setts';
	$nonceStr = isset( $_POST['admin_page'] ) ? 'eos_dp_admin_setts' : $nonceStr;
	eos_dp_check_intentions_and_rights( $nonceStr );
	if( isset( $_POST['plugin_path'] ) ){
		if( isset( $_POST['admin_page'] ) && esc_url( $_POST['admin_page'] ) === $_POST['admin_page'] ){
			$admin_page_key = sanitize_key( 
				str_replace( '.','-',
					str_replace( 'admin.php?page=','eos_dp_tlp-',
						str_replace( admin_url(),'',$_POST['admin_page'] ) 
					)
				) 
			);
			set_transient( 'fdp_test_'.$admin_page_key,sanitize_text_field( $_POST['plugin_path'] ),60 );			
		}
		if( isset( $_POST['post_id'] ) && absint( $_POST['post_id'] ) > 0 ){
			set_transient( 'fdp_test_'.sanitize_key( $_POST['post_id'] ),sanitize_text_field( $_POST['plugin_path'] ),60 );
		}
		if( isset( $_POST['post_type'] ) && '' !== $_POST['post_type'] ){
			set_transient( 'fdp_test_'.sanitize_key( $_POST['post_type'] ),sanitize_text_field( $_POST['plugin_path'] ),60 );
		}
		if( isset( $_POST['tax'] ) && '' !== $_POST['tax'] ){
			set_transient( 'fdp_test_'.sanitize_key( $_POST['tax'] ),sanitize_text_field( $_POST['plugin_path'] ),60 );
		}
		if( isset( $_POST['page_speed_insights'] ) && 'true' === $_POST['page_speed_insights'] ){
			set_transient( 'fdp_psi_nonce_'.sanitize_key( $_POST['post_id'],60 ),1000*( absint( time()/1000 ) ) );
		}		
	}
	echo 1;
	die();
	exit;
}
add_action( "wp_ajax_eos_dp_updated_key_for_preview", "eos_dp_updated_key_for_preview" );
//Update the key for preview. Called every 2 minutes
function eos_dp_updated_key_for_preview(){
	eos_dp_check_intentions_and_rights( 'eos_dp_key' );
	echo 1000*absint( time()/1000 );
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_check_single_padlock', 'eos_dp_check_single_padlock' );
//Check if post type overrides single settings
function eos_dp_check_single_padlock(){
	eos_dp_check_intentions_and_rights( 'eos_dp_setts' );
	if( isset( $_POST['post_type'] ) ){
			$plugins_table = eos_dp_plugins_table();
			$arr = $plugins_table[sanitize_key( $_POST['post_type'] )];
			echo !isset( $arr[0] ) || $arr[0] == '1'  ? 'override' : 'not-override';
			die();
			exit;
	}
	die();
	exit;
}
	
	
add_action( 'wp_ajax_eos_dp_pro_auto_settings', 'eos_dp_pro_auto_settings' );
add_action( 'wp_ajax_nopriv_eos_dp_pro_auto_settings', 'eos_dp_pro_auto_settings' );
//Auto settings
function eos_dp_pro_auto_settings( $post_args = false,$plugins = false ){
	$post_args = $post_args ? $post_args : $_POST;
	$internal_call = isset( $post_args['internal_call'] ) && true === $post_args['internal_call'];
	$cron = $internal_call ? 'cron_' : '';
	eos_dp_check_intentions_and_rights( 'eos_dp_pro_auto_settings' );
	if ( !isset( $post_args['post_id'] ) && !isset( $post_args['post_type'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	$time0 = time();
	$maxExeTime = absint( ini_get( 'max_execution_time' ) );
	$changedMaxExeTime = false;
	$offset = isset( $post_args['offset'] )? absint( $post_args['offset'] ) : 0;
	if( $maxExeTime < 600 && $maxExeTime > 0){
		@ini_set( 'max_execution_time',600 );
		$changedMaxExeTime = true;
	}
	$args = array( 
		'eos_dp_debug' => 'no_errors',
		'eos_dp_preview' => 'true',
		'eos_dp_offset' => $offset,
		'eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),
		'test_id' => 10000*microtime(1),
		'site_in_progress' => substr( md5( esc_attr( get_option( 'comingsoon_input_psw_email' ) ).'_' ),0,8 )
	);	
	if( isset( $post_args['post_id'] ) ){
		$post_id = absint( $post_args['post_id'] );
		$permalink = get_permalink( $post_id );
		$transient_name = 'fdp_test_'.$cron.$post_id.$offset;
		$args['fdp_post_id'] = $post_id;
	}
	elseif( isset( $post_args['post_type'] ) ){
		$permalink = get_post_type_archive_link( esc_attr( $post_args['post_type'] ) );
		$transient_name = 'fdp_test_'.$cron.sanitize_key( $post_args['post_type'] );
		$args['fdp_post_type'] = esc_attr( $post_args['post_type'] );
	}
	set_transient( $transient_name,'',600 );
	set_site_transient( 'eos_dp_pro_scanning_unused_plugins'.$cron,'true' );
	require_once EOS_DP_PLUGIN_DIR.'/admin/eos-dp-plugins-info.php';
	$plugins_to_skip = array_merge( $plugins_to_skip,$backend_plugins );
	if( !$plugins && isset( $post_args['plugins'] ) ){
		$plugins = explode( ',',esc_attr( $post_args['plugins'] ) );
	}
	$plugins = !$plugins ? eos_dp_active_plugins() : $plugins;
	if( $plugins ){
		$plugins = array_slice( $plugins,$offset,4 );
		$unused_plugins = array();
		@ini_set( 'display_errors',0 );
		if( $internal_call ){
			$args['internal_call'] = 'true';
		}
		$url = urldecode( add_query_arg( $args,wp_nonce_url( $permalink,'eos_dp_preview','eos_dp_preview' ) ) );
		$body_all_plugins_Count = false;
		if( isset( $post_args['post_id'] ) ){
			if( 0 === $offset ){
				delete_site_transient( 'eos_dp_all_count_'.$post_id );
			}
			$body_all_plugins_Count = get_site_transient( 'eos_dp_all_count_'.$post_id );
			if( !$body_all_plugins_Count ){
				$body_all_plugins_Count = eos_dp_pro_count_by_url( $url,'all' );
				set_site_transient( 'eos_dp_all_count_'.$post_id,absint( $body_all_plugins_Count ),600 );
			}
		}
		$body_all_plugins_Count = $body_all_plugins_Count ? absint( $body_all_plugins_Count ) : eos_dp_pro_count_by_url( $url,'all' );
		$n = 0;
		foreach( $plugins as $plugin ){
			if( !in_array( $plugin,$plugins_to_skip ) ){
				$paths_var = ';pn:'.implode( ';pn:',array( $plugin ) );
				set_transient( $transient_name,$paths_var,600 );
				$args['test_id'] = 10000*microtime(1);
				$url = urldecode( add_query_arg( $args,wp_nonce_url( $permalink,'eos_dp_preview','eos_dp_preview' ) ) );
				$bodyCount = eos_dp_pro_count_by_url( $url,$plugin );
				if( 'error' === $bodyCount ){
					break;
				}
				if( $bodyCount > 2 && $bodyCount === $body_all_plugins_Count ){
					$unused_plugins[] = $plugin;
				}
			}
			elseif( in_array( $plugin,$backend_plugins ) ){
					$unused_plugins[] = $plugin;
			}
			++$n;
		}
		if( !$internal_call ){
			echo json_encode( $unused_plugins );
		}
	}
	delete_transient( $transient_name );
	delete_site_transient( 'eos_dp_pro_scanning_unused_plugins'.$cron );
	$display_errors = defined( 'WP_DEBUG_DISPLAY' ) && WP_DEBUG_DISPLAY;
	if( $changedMaxExeTime && $maxExeTime > 0 ){
		@ini_set( 'max_execution_time',$maxExeTime );
	}	
	@ini_set( 'display_errors',$display_errors );
	if( $internal_call && $n === count( $plugins ) ){
		return $unused_plugins;
	}
	if( $n !== count( $plugins ) ){
		echo 'error';
	}
	die();
	exit;
}

add_action( 'wp_ajax_eos_dp_pro_auto_settings_admin', 'eos_dp_pro_auto_settings_admin' );
//Auto settings
function eos_dp_pro_auto_settings_admin( $post_args = false,$plugins = false ){
	$post_args = $post_args ? $post_args : $_POST;
	$offset = isset( $post_args['offset'] )? absint( $post_args['offset'] ) : 0;
	eos_dp_check_intentions_and_rights( 'eos_dp_pro_auto_settings_admin' );
	$internal_call = isset( $post_args['internal_call'] ) && true === $post_args['internal_call'];
	$time0 = time();
	$maxExeTime = absint( ini_get( 'max_execution_time' ) );
	$changedMaxExeTime = false;
	if( $maxExeTime < 300 && $maxExeTime > 0){
		@ini_set( 'max_execution_time',300 );
		$changedMaxExeTime = true;
	}
	$url = esc_url( $post_args['admin_page'] );
	$admin_page_key = sanitize_key( 
		str_replace( '.','-',
			str_replace( 'admin.php?page=','eos_dp_tlp-',
				str_replace( admin_url(),'',$post_args['admin_page'] ) 
			)
		) 
	);
	set_transient( 'fdp_test_'.$admin_page_key,'',300 );
	set_site_transient( 'eos_dp_pro_scanning_unused_plugins_admin','true' );
	$plugins_to_skip = array(
		'freesoul-deactivate-plugins-pro/freesoul-deactivate-plugins-pro.php',
		'freesoul-deactivate-plugins/freesoul-deactivate-plugins.php'
	);	
	if( !$plugins && isset( $post_args['plugins'] ) ){
		$plugins = explode( ',',esc_attr( $post_args['plugins'] ) );		
	}	
	$plugins = !$plugins ? eos_dp_active_plugins() : $plugins;
	if( $plugins ){
		$plugins = array_slice( $plugins,$offset,4 );
		$unused_plugins = array();		
		@ini_set( 'display_errors',0 );
		$args = array( 
			'eos_dp_debug' => 'no_errors',
			'admin_page_key' => $admin_page_key,
			'eos_dp_pro_id' => md5( EOS_DP_PRO_TESTING_UNIQUE_ID ),
			'test_id' => time(),
			'site_in_progress' => substr( md5( esc_attr( get_option( 'comingsoon_input_psw_email' ) ).'_' ),0,8 )
		);
		$url = str_replace( '&amp;','&',add_query_arg( $args,wp_nonce_url( $url,'eos_dp_preview','eos_dp_preview' ) ) );
		$body_all_plugins_Count = eos_dp_pro_count_by_url( $url,'all',true );
		$n = 0;
		$unused_plugins = array();
		foreach( $plugins as $plugin ){
			if( !in_array( $plugin,$plugins_to_skip ) ){
				$paths_var = ';pn:'.implode( ';pn:',array( $plugin ) );
				set_transient( 'fdp_test_'.$admin_page_key,$paths_var,300 );
				$args['test_id'] = time();
				$url = str_replace( '&amp;','&',add_query_arg( $args,wp_nonce_url( $url,'eos_dp_preview','eos_dp_preview' ) ) );
				$bodyCount = eos_dp_pro_count_by_url( $url,$plugin,true );
				if( $bodyCount > 2 && $bodyCount === $body_all_plugins_Count ){
					$unused_plugins[] = $plugin;
				}
			}
			++$n;
		}
		if( $internal_call && $n === count( $plugins ) ){
			return $unused_plugins;
		}
		echo json_encode( $unused_plugins );
		die();
		exit;
	}		
	delete_site_transient( 'eos_dp_pro_scanning_unused_plugins_admin' );
	$display_errors = defined( 'WP_DEBUG_DISPLAY' ) && WP_DEBUG_DISPLAY;
	if( $changedMaxExeTime && $maxExeTime > 0 ){
		@ini_set( 'max_execution_time',$maxExeTime );
	}	
	@ini_set( 'display_errors',$display_errors );
	die();
}

//It retrieves the body html given the url
function eos_dp_pro_count_by_url( $url,$plugin,$admin = false ){
	$start = microtime(1);
	$args = array( 'timeout' => 15 );
	if( $admin ){
		$cookies = array();
		foreach ( $_COOKIE as $name => $value ) {
			$cookies[sanitize_key( $name )] = sanitize_text_field( $value );
		}
		$args['cookies'] = $cookies;
	}
	$response = wp_remote_get( $url,$args );
	if ( !is_wp_error( $response ) ){
		$html = wp_remote_retrieve_body( $response );
		if( !$html || false === strpos( $html,'<' ) ) return false;
		$html = preg_replace( "/<!--(.|\\s)*?-->/","",$html );
		$replace = array(
			"/\/\*(.*?)\*\//" => '',
			"/\/\*(?:(?!\*\/)[\s\S])*\*\/|[\r\n\t]+/" => '',
			"/<!--(.*)-->/Uis" => '',
			"\n" => '',
			"\t" => '',
			"\r" => '',
			"  " => ' ',
			"> <" => '><',
		);
		$search = array_keys( $replace );
		$html = str_replace( $search, $replace, $html );
		if ( !is_wp_error( $html ) ) {
			$metaN = 0;
			libxml_use_internal_errors( true );
			$dom = new DOMDocument();
			$dom->loadHTML( $html );
			if( null === $dom ) return;
			$metas = $dom->getElementsByTagName( 'meta' );
			foreach( $metas as $meta ){
				if( $meta->hasAttribute('name' ) ){
					$metaName = $meta->getAttribute( 'name' );
					if( 
						in_array( $metaName,array(
							'description'
						) )
						|| false !== strpos( $metaName,'og:' )
						|| false !== strpos( $metaName,'twitter:' )
					){
						++$metaN;
					}
				}
			}
			foreach( array( 'head','script','link','style' ) as $key ){
				$remove = array();
				$type = $dom->getElementsByTagName( $key );
				foreach( $type as $item ){
					$remove[] = $item;
				}
				foreach ($remove as $item){
				  $item->parentNode->removeChild($item); 
				}
			}
			if( $admin ){
				$remove = array();
				$ids = array( 'adminmenuwrap','wp-admin-bar-root-default' );
				foreach( $ids as $id ){
					$el = $dom->getElementById( $id );
					if( $el ){
						$el->parentNode->removeChild( $el );
					}
				}
				if( class_exists( 'DomXPath' ) ){
					$finder = new DomXPath( $dom );
					$classname = 'notice';
					$els = $finder->query("//*[contains(concat(' ', normalize-space(@class), ' '), ' $classname ')]");
					foreach( $els as $item ){
						$item->parentNode->removeChild($item); 	
					}
					$type = 'hidden';
					$els = $finder->query("//*[contains(concat(' ', normalize-space(@type), ' '), ' $type ')]");
					foreach( $els as $item ){
						$item->parentNode->removeChild($item); 	
					}
				}
			}
			$body = $dom->getElementsByTagName( 'body' );
			if( $body && isset( $body[0] ) ){
				$body[0]->setAttribute('class', 'none');
				$html = $dom->getElementsByTagName( 'html' );
				$html[0]->setAttribute('class', 'none');	
			}
			$string = $dom->saveHTML();
			$n = substr_count( $string,'<' );
			$n += substr_count( $string,'</' );
			if( !$admin ){
				$n += substr_count( $string,'class="' );
				$n += substr_count( $string,'id="' );
				$n += substr_count( $string,'[' );
				$n += substr_count( $string,'/]' );
				$n += $metaN;
			}
			return $n;			
		}				
	}
	else{
		return 'error';
	}
	return false;
}
add_action( 'wp_ajax_eos_dp_msg_never_again','eos_dp_msg_never_again' );
//It prevent future notifications to the same user
function eos_dp_msg_never_again(){
	eos_dp_check_intentions_and_rights( 'eos_dp_never_again_msg_user' );
	$nonce = isset( $_POST['nonce'] ) ? $_POST['nonce'] : false;
	if ( !isset( $_POST['msg'] ) ){
	   echo 0;
	   die();
	   exit;
	}
	$user = wp_get_current_user();
	if( !is_object( $user ) ){
		die();
		exit;
	}
	$user_opts = get_site_option( 'eos_dp_user_options' );
	$user_opts = is_array( $user_opts ) && !empty( $user_opts ) ? $user_opts : array();
	$user_opts[sanitize_key( $user->user_login )] = array( sanitize_key( $_POST['msg'] ) => 'never_again' );
	update_site_option( 'eos_dp_user_options',$user_opts );
	die();
	exit;	
}

//Check for intentions and rights
function eos_dp_check_intentions_and_rights( $nonce_action ){
	if( isset( $_REQUEST['eos_dp_advanced_setup'] ) || isset( $GLOBALS['eos_dp_advanced_setup'] ) ){
		$eos_dp_advanced_setup = isset( $GLOBALS['eos_dp_advanced_setup'] ) ? $GLOBALS['eos_dp_advanced_setup'] : $_REQUEST['eos_dp_advanced_setup'];
		$opts = eos_dp_get_option( 'eos_dp_opts' );
		$password = isset( $opts['advanced_help_password'] ) ? esc_attr( $opts['advanced_help_password'] ): '';
		if( '' !== $password && strlen( $password ) > 9 && $password === $eos_dp_advanced_setup ){
			return;
		}
	}
	$nonce = isset( $_POST['nonce'] ) ? $_POST['nonce'] : false;
	if (
		false === $nonce
		|| ! wp_verify_nonce( $nonce,$nonce_action ) //check for intentions
		|| !current_user_can( 'activate_plugins' ) //check for rights
	) {
	   echo 0;
	   die();
	   exit;
	}
}

//Convert string size to numeric size
function eos_let_to_num( $size ) {
	$l   = substr( $size, -1 );
	$ret = substr( $size, 0, -1 );
	switch ( strtoupper( $l ) ) {
		case 'P':
			$ret *= 1024;
		case 'T':
			$ret *= 1024;
		case 'G':
			$ret *= 1024;
		case 'M':
			$ret *= 1024;
		case 'K':
			$ret *= 1024;
	}
	return $ret;
}